import {
  scaleLinear,
  scaleTime,
  timeFormat,
  extent,
  bin,
  timeMonths,
  sum,
  max,
  brushX,
  select,
} from 'd3';
import { useRef, useEffect, useMemo } from 'react';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const xAxisLabelOffset = 54;
const yAxisLabelOffset = 30;
const margin = { top: 0, right: 30, bottom: 20, left: 45 };
const xAxisTickFormat = timeFormat('%m/%d/%Y');

const xAxisLabel = 'Time';
const yValue = (d) => d['Total Dead and Missing'];
const yAxisLabel = 'Total Dead and Missing';

export const DateHistogram = ({
  data,
  width,
  height,
  setBrushExtent,
  xValue,
}) => {
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(data, xValue))
        .range([0, innerWidth])
        .nice(),
    [data, xValue, innerWidth]
  );

  ////**** Binned Aggregation ****////
  //** 1. Binning **//
  const aggregatedData = useMemo(() => {
    const [start, end] = xScale.domain();
    const binnedData = bin()
      .value(xValue)
      .domain(xScale.domain())
      .thresholds(timeMonths(start, end))(data);
    //console.log("originalData", data);
    //console.log("binnedData", binnedData);

    //** 2. Aggregation (with sum) **//
    const aggregatedBinnedData = binnedData.map(
      (array) => ({
        y: sum(array, yValue),
        x0: array.x0,
        x1: array.x1,
      })
    );
    //console.log(aggregatedBinnedData);
    return aggregatedBinnedData;
  }, [xValue, yValue, xScale, data]);

  const yScale = useMemo(
    () =>
      scaleLinear()
        .domain([0, max(aggregatedData, (d) => d.y)])
        .range([innerHeight, 0]),
    [aggregatedData, innerHeight]
  );

  // Accessing RAW DOM element created by JSX
  // 1. Create an instance of useRef hook
  // 2. Pass useRef instance to a native jsx element as 'ref={}' attr
  // 3. <ref_instance>.current now contains the RAW DOM element
  // 4. Manipulate the DOM element within useEffect hook.
  const brushRef = useRef();
  useEffect(() => {
    const brush = brushX().extent([
      [0, 0],
      [innerWidth, innerHeight],
    ]);
    brush(select(brushRef.current));
    brush.on('brush end', (event) => {
      setBrushExtent(
        event.selection &&
          event.selection.map(xScale.invert)
      );
    });
  }, [innerWidth, innerHeight]);

  return (
    <>
      <rect width={width} height={height} fill="white" />
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={5}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${
            innerHeight / 2
          }) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={5}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <Marks
          data={aggregatedData}
          xScale={xScale}
          yScale={yScale}
          tooltipFormat={(d) => d}
          innerHeight={innerHeight}
        />
        <g ref={brushRef} />
      </g>
    </>
  );
};
